/**
 * \file BusWorkflow.h
 *
 *  \date Apr 12, 2012
 *  \author Alexandre Tuleu
 */

#ifndef LIBSBCP_BUS_BUS_BUSWORKFLOW_H_
#define LIBSBCP_BUS_BUS_BUSWORKFLOW_H_

#include <tr1/memory>

#include <biorob-cpp/base/common.h>

/**
 *
 */
namespace sbcp {
	class Bus;
	/**
	 * Abstarction of operation that can be handled over the sbcp::Bus.
	 *
	 * It represents a schema of consistent operation that could be performed
	 * over the bus. There is two main strategies actually : LazyWorkflow wich
	 * is synchronous., and fast, low latency, asynchronous, but more limited
	 * ScheduledWorkflow.
	 *
	 * Basically a workflow is a state machine for handling operation over the
	 * bus.
	 *
	 * BusWorkflow are for performing low-level I/O. You should prefer to use
	 * more high level RegisterAccessor or device InstructionID implementation
	 * implementation.
	 */
	class BusWorkflow {
		DISABLE_COPY(BusWorkflow)
		DISABLE_ASSIGNEMENT(BusWorkflow)
	public:
		typedef std::tr1::shared_ptr<BusWorkflow> Ptr;

		virtual ~BusWorkflow();
		unsigned int Type() const;
	protected :
		//only subclass and Bus could create a BusWorkflow
		BusWorkflow(Bus & bus, unsigned int type);


	protected :
		Bus & d_bus;
		unsigned int d_type;
	};

}
#endif // LIBSBCP_BUS_BUSWORKFLOW_H_
