/*
 * \file Register.cpp
 *
 *  \date Jun 4, 2013
 *  \author Alexandre Tuleu
 */


#include <sstream>
#include <stdexcept>

#include "Register.h"

#include "Device.h"

#include <libsbcp/utils/HexaByte.h>

namespace sbcp {

Register::Register(Device& device , Address adr , int flags)
	: d_address(adr)
	, d_flags((Flag) (flags & PublicFlagsMask)){

	Device::RegisterAddAccessor::AddRegister(device,this);
	if ( (flags & (~PublicFlagsMask) ) != 0){
		std::ostringstream os;
		os << "Cannot create register " << HexaByte(adr)
		   << " as it sets flags " << HexaByte(flags) << " but only "
		   << HexaByte(PublicFlagsMask) << " are allowed.";
		throw std::logic_error(os.str());
	}
}

sbcp::Register::~Register(){}

Register::Address Register::GetAddress() const{
	return d_address;
}

Register::Flag Register::Flags() const{
	return d_flags;
}

void Register::SetFlags(unsigned int flg){
	d_flags = (Flag) flg;
}

void Register::SetFlag(Flag f){
	d_flags = (Flag) (d_flags |  f);
}

void Register::UnsetFlag(Flag f){
	d_flags = (Flag) (d_flags & ~f);
}


} //namespace sbcp
