/*
 * \file Register.h
 *
 *  \date Jun 4, 2013
 *  \author Alexandre Tuleu
 */

#ifndef LIBSBCP_REGISTER_H_
#define LIBSBCP_REGISTER_H_

#include "sbcp_common.h"

namespace sbcp {

class Device;

class Register {
public :
	/**
	 * Flags of the register.
	 */
	enum Flag{
		INEXISTANT     = 0,     //!< INEXISTANT internal initialization state.
		READABLE       = 1 << 0,//!< READABLE this register is readable outside of
		                        //!<          of the sbcp::Device
		WRITABLE       = 1 << 1,//!< WRITABLE this register is writable outside of
		                        //!<          of the sbcp::Device .
		PERSISTENT     = 1 << 2,//!< PERSISTENT if the device register is persistent in memory
		LL_READABLE    = 1 << 3,
	};

	const static Flag PublicFlagsMask = (Flag) 0x0F;
	typedef uint8_t Address;

	Register(Device & device, Address adr, int flags);
	virtual ~Register();

	Address GetAddress() const;
	Flag   Flags() const;
	void SetFlags(unsigned int flg);
	void SetFlag(Flag f);
	void UnsetFlag(Flag f);
private :
	Address d_address;
	Flag   d_flags;
};

} //namespace sbcp


#endif /* LIBSBCP_REGISTER_H_ */
