/**
 * \file BusConfigSection.h
 *
 *  \date Dec 14, 2012
 *  \author Alexandre Tuleu
 */

#ifndef LIBSBCP_UTILS_BUSCONFIGSECTION_H_
#define LIBSBCP_UTILS_BUSCONFIGSECTION_H_

#include "HexaByte.h"

#include <biorob-cpp/options/Config.h>
#include <libsbcp/bus/Bus.h>



/**
 *
 */
namespace sbcp {
/**
 *
 */
class BusConfigSection;

class SerialInterfaceConstructor {
public :
	virtual ~SerialInterfaceConstructor();
	typedef std::tr1::shared_ptr<SerialInterfaceConstructor> Ptr;
	virtual sbcpd_serial_interface * Create(const BusConfigSection & config) = 0;
};

class BusConfigSection : options::ConfigSection{
public:
	BusConfigSection(options::Section::Ptr s);
	virtual ~BusConfigSection();

	BCPP_ADD_OPTION(Baudrate,"baudrate",unsigned int,3310000,"baudrate of the bus")
	BCPP_ADD_OPTION(SerialType,"serial-type",std::string,"ft2232h","Serial interface to use. Currently only 'linux','ftdi' and 'ft2232h' are supported")
	BCPP_ADD_OPTION(FtdiInterface,"ftdi-interface",unsigned int,2,"ftdi interface to use (ftdi/ft2232honly)")
	BCPP_ADD_OPTION(Latency,"latency",unsigned int,0,"Latency to use (ftdi/ft2232h only)")
	BCPP_ADD_OPTION(DevName,"dev",std::string,"/dev/stty1","Linux device filename, (linux only)")
	BCPP_ADD_OPTION(MasterID,"master-id",HexaByte,0,"ID of the master communication board")
	BCPP_ADD_OPTION(MinFrameSize,"min-frame-size",unsigned int,1,"required number of packet in a frame")
	BCPP_ADD_OPTION(MinPacketSize,"min-packet-size",unsigned int,16,"required size of a packet")

	std::tr1::shared_ptr<Bus> OpenBusWithFrame(unsigned int minFrameSize,
	                                           unsigned int minPacketSize) const;

	std::tr1::shared_ptr<Bus> OpenBus() const;
private :


	typedef std::map<std::string,SerialInterfaceConstructor::Ptr> SerialInterfaceConstructorByName;

	static void InitSiConstructors();

	static SerialInterfaceConstructorByName s_siConstructors;

};


} //namespace sbcp
#endif // LIBSBCP_CONFIGSECTION_H_
