/**
 * \file Config.cpp
 *
 *  \date Dec 14, 2012
 *  \author Alexandre Tuleu
 */

#include "Config.h"

namespace sbcp {
	const char * BusesConfigSection::DEFAULT_BUS_NAME = "default";

	BusesConfigSection::BusesConfigSection(options::Section::Ptr s)
		: options::DynamicConfigSection<BusConfigSection>(s){
		Section().AddSubSection(DEFAULT_BUS_NAME,"default bus definition");
	}

	BusesConfigSection::~BusesConfigSection(){
	}

	const BusConfigSection & BusesConfigSection::Bus(const std::string & name) const{
		typedef std::tr1::shared_ptr<BusConfigSection> Ptr;
		//todo remove the const cast on biorob-cpp upstream fix
		Ptr b = const_cast<BusesConfigSection*>(this)->SubSection(name);
		if(!b){
			throw std::out_of_range("Could not find bus '" + name + "'");
		}
		return *b;
	}

	const BusConfigSection & BusesConfigSection::Default() const{
		return Bus(DEFAULT_BUS_NAME);
	}

	Config::Config()
		: options::Config("sbcp")
		, d_shBusDefinitions(Section()){
	}

	Config::~Config(){
	}

	std::tr1::shared_ptr<Bus> Config::OpenDefaultBus() const {
		return BusDefinitions().Default().OpenBus();
	}

	std::tr1::shared_ptr<Bus> Config::OpenDefaultBusWithFrame(unsigned int requiredFrameSize,
	                                                          unsigned int requiredPacketSize) const {
		return BusDefinitions().Default().OpenBusWithFrame(requiredFrameSize,requiredPacketSize);
	}


	std::tr1::shared_ptr<Bus> Config::OpenBus(const std::string & name) const {
		return BusDefinitions().Bus(name).OpenBus();
	}

	std::tr1::shared_ptr<Bus> Config::OpenBusWithFrame(const std::string & name,
	                                                   unsigned int requiredFrameSize,
	                                                   unsigned int requiredPacketSize) const {
		return BusDefinitions().Bus(name).OpenBusWithFrame(requiredFrameSize,requiredPacketSize);
	}

} /* namespace sbcp */
