/**
 * \file Config.h
 *
 *  \date Dec 14, 2012
 *  \author Alexandre Tuleu
 */

#ifndef LIBSBCP_UTILS_CONFIG_H_
#define LIBSBCP_UTILS_CONFIG_H_

#include "BusConfigSection.h"

/**
 *
 */
namespace sbcp {


class BusesConfigSection : public options::DynamicConfigSection<BusConfigSection>{
public :
	const static char * DEFAULT_BUS_NAME;
	BusesConfigSection(options::Section::Ptr s);
	virtual ~BusesConfigSection();

	const BusConfigSection & Bus(const std::string & name) const;
	const BusConfigSection & Default() const;
};

/**
 *
 */
class Config : public options::Config{
public:
	Config();
	virtual ~Config();

	BCPP_ADD_SUBSECTION(BusesConfigSection,BusDefinitions,"bus","Definitions of all available buses");
	/**
	 * Opens the default bus defined on the system.
	 * @return a shared pointer to the newly opened bus.
	 */
	std::tr1::shared_ptr<Bus> OpenDefaultBus() const;
	/**
	 * Opens the default bus defined on the system, overriding the frame
	 * definition. Equivalent to OpenBus("default").
	 * @param requiredFrameSize the minimum frame size
	 * @param requiredPacketSize the minimu packet size
	 * @return a shared pointer to the newly opened bus.
	 */
	std::tr1::shared_ptr<Bus> OpenDefaultBusWithFrame(unsigned int requiredFrameSize,
	                                                  unsigned int requiredPacketSize) const;

	/**
	 * Opens the named bus of the system
	 * @param name the name of the bus
	 * @return a shared pointer to the newly opened bus.
	 */
	std::tr1::shared_ptr<Bus> OpenBus(const std::string & name) const;

	/**
	 * Opens the named bus defined on the system, overriding the frame
	 * definition.
	 * @param requiredFrameSize the minimum frame size
	 * @param requiredPacketSize the minimu packet size
	 * @return a shared pointer to the newly opened bus.
	 */
	std::tr1::shared_ptr<Bus> OpenBusWithFrame(const std::string & name,
	                                           unsigned int requiredFrameSize,
	                                           unsigned int requiredPacketSize) const;


};
} //namespace sbcp
#endif // LIBSBCP_CONFIGF_H_
