/*
 * HexaByte.cpp
 *
 *  Created on: Jan 4, 2013
 *      Author: tuleu
 */

#include "HexaByte.h"

#include <sstream>
#include <stdexcept>

namespace sbcp {

HexaByte::HexaByte()
	: d_byte(UNITIALIZED){
}

HexaByte::HexaByte(uint8_t v)
	: d_byte(v){
}

HexaByte::~HexaByte(){
}

void HexaByte::operator=(uint8_t v){
	d_byte = v;
}

HexaByte::operator uint8_t() const {
	return d_byte & 0xff;
}


bool HexaByte::IsInitialized() const{
	return d_byte != UNITIALIZED;
}


std::ostream & operator << (std::ostream & out, const HexaByte & b){
	std::ios_base::fmtflags oldflags = out.flags();
	out << "0x" << std::hex << (b.d_byte & 0xff);
	out.flags(oldflags);
	return out;
}


std::istream & operator >> (std::istream & in, HexaByte & b){
	std::ios_base::fmtflags oldflags = in.flags();
	unsigned int value;
	in >> std::hex >> value;
	if (value > 0xff){
		std::ostringstream os;
		os << "0x" << std::hex << value << " does not fit in a 8-bit byte.";
		throw std::out_of_range(os.str());
	}
	b.d_byte = value;
	in.flags(oldflags);
	return in;
}

} /* namespace sbcp */
